/*  $Id: dig_process.pl, v1.4 2005/2/24  zhisheng Exp $


    Author:        Zhisheng Huang
    E-mail:        huang@cs.vu.nl
    WWW:           http://www.cs.vu.nl/~huang
    Copyright (C): 2004 Vrije University Amsterdam

This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    As a special exception, if you link this library with other files,
    compiled with a Free Software compiler, to produce an executable, this
    library does not by itself cause the resulting executable to be covered
    by the GNU General Public License. This exception does not however
    invalidate any other reasons why the executable file might be covered by
    the GNU General Public License.

*/


:- module(dig_process,
	  [ dig_add_elements/3,			% +Data, +Type, -ElementsWithType
	    xml_elements/3,			% +RawText, -XMLElements, -Header
	    elements_xmltext/2,			% +XMLElements, -XMLText
	    elements_xmltext_text/2,		% +XMLElements, -XMLText  %% version 1.1
	    xmltext_elements/2,			% +XMLText, -XMLElements, %% version 1.1
	    dig_requestdata_analysis/3,	%+RequestData, -Data, -Type
	    dig_data_analysis/4,		%+Data, +Type, -Element, -OtherData
	    dig_response_analysis/4,		%+Data, +Type, -Element, -OtherData
	    dig_select_attribute/3, 		% +AttributeList, +Type, -Message, %% version 1.1
	    dig_get_element/3,			% +List, +Element, -Other,   %% version 1.1
	    dig_xmlns/1,				%-XMLNS List
	    dig_tmp_working_file/2		% +Type, -File
	  ]).



:-use_module(library('memfile')).	%% internal memory file
:-use_module(library('sgml')).

:- dynamic
	tmp_working_file/2.



xml_elements(RawText, Elements, Header) :-
	atom_codes(RawText, RawCodes),
	skip_until_xml(RawCodes, XMLCodes, [], Header),
	atom_codes(XMLText, XMLCodes),
	xmltext_elements(XMLText, Elements).

xmltext_elements(XMLText, Elements):-
	atom_to_memory_file(XMLText, MemoryHandle),
	open_memory_file(MemoryHandle, read, MemoryStream),
	load_structure(MemoryStream, Elements, []),    
	close(MemoryStream),
	free_memory_file(MemoryHandle).



skip_until_xml([], [], Header, Header).
skip_until_xml(CI, CO, Header, Header) :-
	CI = [0'<, 0'?, 0'x, 0'm, 0'l | _],
	!,
	keep_xml_codes(CI, CO).
skip_until_xml([H|T], CO, OldHeader, Header) :-
	append(OldHeader, [H], NewHeader),
	skip_until_xml(T, CO, NewHeader, Header).

keep_xml_codes([], []) :-
	!.
keep_xml_codes(CI, CO) :-
	CI = [0'r, 0'e, 0's, 0'p, 0'o, 0'n, 0's, 0'e, 0'> | _],
	CO = [0'r, 0'e, 0's, 0'p, 0'o, 0'n, 0's, 0'e, 0'>],
	!.
keep_xml_codes([HI|TI], [HI|TO]) :-
	keep_xml_codes(TI, TO).


get_header_value([], reply_code, code('cannot find the http head')):-!.
get_header_value(CI, reply_code, code(Code)):-
		CI = [0'H, 0'T, 0'T, 0'P, 0'/, _Major, 0'., _Minor, _Space, Code1, Code2, Code3 | _],
		!,
		CodeList = [Code1, Code2, Code3],
            atom_codes(Code, CodeList).

get_header_value([_H|T], reply_code, Code):-
		get_header_value(T, reply_code, Code).

% elements_xmltext(+Elements, XMLText)



elements_xmltext(Elements, file(XMLFile)):-
		catch(open(XMLFile, write, Out, []), error(permission_error(open,source_sink,_), _), (sleep(3), elements_xmltext(Elements, file(XMLFile)))),  
%		open(XMLFile, write, Out, []),
		print_xml_head(Out),
		elements_xml(Elements, Out),
            close(Out).

elements_xmltext(Elements, stream(Out)):-
		print_xml_head(Out),
		elements_xml(Elements, Out).

%% version 1.1
elements_xmltext(Elements, text(Text)):-
       elements_xml_text(Elements, Text).	


print_xml_head(Out):-
		write(Out, '<?xml version="1.0" encoding="UTF-8"?>').


elements_xml(L,  Out):-
   elements_xml_text(L, Text),
   format(Out, '~w', [Text]).


dig_add_elements(Elements, tells, TellElements):-
                   !,
			dig_xmlns(XMLNS),
			TellElements = [element(tells, XMLNS, Elements)].

dig_add_elements(Elements, asks, TellElements):-
                  !,
			dig_xmlns(XMLNS),
			TellElements = [element(asks, XMLNS, Elements)].

dig_add_elements(Elements, clearKB, NewElements):-
                  !,
			NewElements = [element(clearKB, [],[])|Elements].

dig_add_elements(Elements, Type, NewElements):-
			NewElements = [element(Type, [], Elements)].



dig_requestdata_analysis(RequestData, Data, Type):-
			RequestData = elements(Elements),
			Elements = [element(Type, _, Data)].


dig_requestdata_analysis(RequestData, Data, Type):-
			RequestData = text(XMLText),
			xml_elements(XMLText, Elements, _Header),
			Elements = [element(Type, _, Data)].


dig_requestdata_analysis(RequestData, RequestData, tell):-!.

dig_data_analysis(Data, Type, Element, OtherData):-
		Element =element(Type, _, _),
		select(Element, Data, OtherData).


dig_data_analysis(Data, _Type, nil, Data):-!.

dig_response_analysis(AnswerBody, Type, Element, Other):-
			AnswerBody = [element(response, _, Elements)|_],
			Element=element(Type,_,_),
			select(Element, Elements, Other).

dig_response_analysis(AnswerBody, Type, Element, Other):-
			AnswerBody = [element(responses, _, Elements)|_],
			Element=element(Type,_,_),
			select(Element, Elements, Other).

dig_response_analysis(_,_,nil,_):-!.


dig_tmp_working_file(txt, 'tmp_xml.txt').
dig_tmp_working_file(xml, 'tmp_xml.xml').

dig_xmlns([xmlns='http://dl.kr.org/dig/lang', xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance', xsi:schemaLocation='http://wasp.cs.vu.nl/sekt/dig/dig-1.1.xsd']):-!.



%% version 1.1
dig_get_element([], nil, []):-!.
dig_get_element([E|T], E1, T):-
     E= element(_,_,_),
     !,
     E1 = E.

dig_get_element([_E|T],E,L):-
     !,
     dig_get_element(T,E,L).




dig_select_attribute([],_, nil):-!.
dig_select_attribute([Type=M|_T], Type, M):-!.
dig_select_attribute([_|T], Type, M):-
   dig_select_attribute(T, Type, M).

elements_xmltext_text(Elements, Text):-
		print_xml_head_text(Text1),
		elements_xml_text(Elements, Text2),
            concat_atom([Text1, Text2], Text).


	

print_xml_head_text(Text):-
		sformat(Text, '<?xml version="1.0" encoding="UTF-8"?>', []).



elements_xml_text([], ' '):-!.

elements_xml_text([H|T], Text):-
	element_xml_text(H, Text1),
	!,
	elements_xml_text(T, Text2),
      concat_atom([Text1, Text2], Text).



element_xml_text(element(Node, [], []), Text):-!,
       sformat(Text, '<~w/>',  [Node]).

element_xml_text(element(Node, Attrib, []), Text):-!,
       sformat(Text1, '<~w',  [Node]),
       print_all_attributes_text(Text2, Attrib),
	 sformat(Text3, '/>', []),
       concat_atom([Text1, Text2, Text3], Text).
	 
		
element_xml_text(element(Node, Attributes, Elements), Text):-!,
       sformat(Text1, '<~w',  [Node]),
	 print_all_attributes_text(Text2, Attributes),
       sformat(Text3, '>',[]),
	 elements_xml_text(Elements, Text4),
	 sformat(Text5, '</~w>', [Node]),
       concat_atom([Text1, Text2, Text3, Text4, Text5],Text).
       

element_xml_text(X, Text):-
     delete_code(X, 13, Text1),
     delete_code(Text1, 10, Text).

element_xml_text(X, X):-!.
% capture all statement to avoid the error in the predicate.
% new added for version 1.4

		
     
print_all_attributes_text('', []):-!.
print_all_attributes_text(Text, [H=Value|T]):-
		sformat(Text1, ' ~w = "~w"', [H, Value]),
		!,
		print_all_attributes_text(Text2, T),
            concat_atom([Text1, Text2], Text).


print_all_attributes_text(Text, [H|T]):-
		sformat(Text1, ' ~q', [H]),
		!,
		print_all_attributes_text(Text2, T),
            concat_atom([Text1, Text2], Text).



delete_code(Text, Code, Newtext):-
		atom_codes(Text, Codes),
		delete(Codes, Code, NewCodes),
		atom_codes(Newtext, NewCodes).

