/*  $Id: dig_server.pl, v 1.2 2004/12/02  zhisheng Exp $


    Author:        Zhisheng Huang
    E-mail:        huang@cs.vu.nl
    WWW:           http://www.cs.vu.nl/~huang
    Copyright (C): 2004 Vrije University Amsterdam

This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    As a special exception, if you link this library with other files,
    compiled with a Free Software compiler, to produce an executable, this
    library does not by itself cause the resulting executable to be covered
    by the GNU General Public License. This exception does not however
    invalidate any other reasons why the executable file might be covered by
    the GNU General Public License.

*/


:- module(dig_server,
	  [ dig_server_port/1,			% -Port
	    dig_server_id/1,			%-DIGServerID
	    set_dig_server_port/1,		% +Port
	    set_dig_server_id/1,		% +DIGServerID
	    dig_standard_response/3,		%+Type, +ID, -Answer
	    dig_server/1			% +Request
	  ]).



:-use_module(library('dig/dig_process')).
:-use_module(library('http/http_open')).
:-use_module(library('http/thread_httpd')).
:-use_module(library('http/http_client')).
:-use_module(library('memfile')).	%% internal memory file
:-use_module(library('sgml')).

:- dynamic
	dig_server_id/1,
	dig_server_port/1.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



dig_server(Request):-  
			member(method(post), Request),
			member(content_type('text/xml'), Request),
			!,
			http_read_data(Request, Data, [to(atom), connection(close)]),
		      xmltext_elements(Data, XML),	
			my_dig_server_processing(elements(XML), Answer, [connection(close)]),
			dig_server_reply(Request, Answer, xml),
			true.


dig_server(Request):-  
			member(method(post), Request),
			member(content_type('application/x-www-form-urlencoded'), Request),
			!,
			http_read_data(Request, Data, [connection(close)]),
			get_form_data(Data, dig_xml_data, XMLText),
                  xmltext_elements(XMLText, XML),	
			my_dig_server_processing(elements(XML), Answer, [connection(close)]),
			dig_server_reply(Request, Answer, html),
			true.


dig_server(Request):-  
			!,
			format('Content-type: text/plain~n~n',[]),
			dig_header_reply(Request, 400),
			true.


get_form_data([], _, ''):-!.

get_form_data([Name = Value|_T], Name, Value):-!.


get_form_data([_Name = _Value|T], Name, Value):-
		get_form_data(T, Name, Value).




dig_server_reply(Request, Answer, xml):-
			Answer = answer(_H, text(Text)),
			format('Content-type: text/plain~n~n',[]),
			dig_header_reply(Request,200),
			format('~w~n',[Text]),
			true.		



dig_server_reply(_Request, Answer, html):-
			Answer = answer(_H, text(Text)),
			format('Content-type: text/xml~n~n',[]),
			format('~w~n',[Text]),
			true.

dig_server_reply(Request, Answer, xml):-
			Answer = answer(_H, Reply),
			format('Content-type: text/plain~n~n',[]),
			dig_header_reply(Request,200),
			elements_xmltext(Reply, stream(current_output)),
			true.

dig_server_reply(_Request, Answer, html):-
			Answer = answer(_H, Reply),
			format('Content-type: text/xml~n~n',[]),
                  dig_get_element(Reply, E, _),
			elements_xmltext([E], stream(current_output)),
			true.

dig_header_reply(Request, Code):-
			select(host(Host), Request, Request1),
			select(path(Path), Request1, _Request2),
			get_time(Time),
			convert_time(Time, Date),
			dig_status_code(Code, CodeText),
			format('HTTP/1.1 ~w ~w~n',[Code, CodeText]),  
			format('Date: ~w~n',[Date]),
			dig_server_id(ServerID),
			format('Server: ~w~n',[ServerID]),
			format('Connection: close~n', []),
			format('Content-type: text/xml~n',[]),
			format('Content-Location: http://~w~w~n',[Host, Path]),
			format('Transfer-Encoding: chunked~n~n',[]).

dig_header_reply(_Request, _Code):-!.

set_dig_server_port(Port):-
			retractall(dig_server_port(_)),
			assert(dig_server_port(Port)).

set_dig_server_id(ID):-
			retractall(dig_server_id(_)),
			assert(dig_server_id(ID)).

dig_server_port(8001).
dig_server_id('XDIG Server (1.2.0)').




dig_standard_response(ok, _ID, Answer):-
		Header = 'HTTP/1.1 200 OK~nContent-type: text/xml~n~n',
		sformat(Reply, '<?xml version="1.0" encoding="ISO-8859-1"?><response xmlns="http://dl.kr.org/dig/lang" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://wasp.cs.vu.nl/sekt/dig/dig-1.1.xsd"><ok/></response>',[]),
		Answer = answer(Header, text(Reply)).

dig_standard_response(true, ID, Answer):-
		Header = 'HTTP/1.1 200 OK~nContent-type: text/xml~n~n',
		sformat(Reply, '<?xml version="1.0" encoding="ISO-8859-1"?><response xmlns="http://dl.kr.org/dig/lang" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://wasp.cs.vu.nl/sekt/dig/dig-1.1.xsd"><true id="~w"/></response>',[ID]),
		Answer = answer(Header, text(Reply)).

dig_standard_response(false, ID, Answer):-
		Header = 'HTTP/1.1 200 OK~nContent-type: text/xml~n~n',
		sformat(Reply, '<?xml version="1.0" encoding="ISO-8859-1"?><response xmlns="http://dl.kr.org/dig/lang" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://wasp.cs.vu.nl/sekt/dig/dig-1.1.xsd"><false id="~w"/></response>',[ID]),
		Answer = answer(Header, text(Reply)).




dig_status_code(200, 'OK'):-!.
dig_status_code(204, 'No Content'):-!.
dig_status_code(400, 'Bad Request'):-!.
dig_status_code(401, 'Unauthorized'):-!.
dig_status_code(403, 'Forbidden'):-!.
dig_status_code(405, 'Method Not Allowed'):-!.
dig_status_code(408, 'Request Time-out'):-!.
