/*  $Id: pion_selection.pl, v1.1 2005/2/23  zhisheng Exp $


    Author:        Zhisheng Huang
    E-mail:        huang@cs.vu.nl
    WWW:           http://www.cs.vu.nl/~huang
    Copyright (C): 2004 Vrije University Amsterdam

This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    As a special exception, if you link this library with other files,
    compiled with a Free Software compiler, to produce an executable, this
    library does not by itself cause the resulting executable to be covered
    by the GNU General Public License. This exception does not however
    invalidate any other reasons why the executable file might be covered by
    the GNU General Public License.

*/

/* 
this library need the xdig package (version 1.1, or later)
*/


:- module(pion_selection,
	  [pion_selection_function/5 	%+SelectionFunctionType,  +QueryElement, +WorkingSet, +KBID, -SelectedSet
	  ]).


pion_selection_function(Type, Q, P, KBID, L):-
      Type=syntactic,
      !,
      icr_set_element(Q, icr(IQ, CQ, RQ)),
      icr_set(P, icr(IP, CP, RP)),
      union(IQ, IP, I),
	union(CQ, CP, C),
	union(RQ, RP, R),
	selected_set(Type, icr(I,C,R), P, KBID, L).

pion_selection_function(Type, _Q, _P, _KBID, _L):-
      format('undefined selection function type: ~w~n',[Type]).


%selected_set(+ICRSetofCurrentWorkingSet, +CurrentWorkingSet, +KBID, -SelectedSet)
selected_set(syntactic, ICR, P, KBID, L):-
 %%   setof(E, (dig_db_element(KBID, E),\+ member(E, P), relevant(E, ICR)), L), % this line changes the order of the .
	bagof(E, (dig_db_element(KBID, E),\+ member(E, P), relevant(E, ICR)), L1),
      list_to_set(L1, L),
%%    the above line is added  to avoid the duplicated elements in L (version 1.1)
      true.



%relevant(ICR1, ICR2): two ICR are relevant
relevant(icr(IE,CE, RE), icr(I,C, R)):-
	(intersection(I, IE, [_I|_]); intersection(C, CE, [_C|_]); intersection(R, RE, [_R|_])),!.

%relevant(Element, ICR): the element is relevant to the ICR.
relevant(E, icr(I,C,R)):-
	icr_set_element(E, icr(IE, CE, RE)),
	(intersection(I, IE, [_I|_]); intersection(C, CE, [_C|_]); intersection(R, RE, [_R|_])).

%get the -ICR for an element +Set 
icr_set([], icr([],[],[])):-!.
icr_set([H|T], icr(I2,C2, R2)):-
			icr_set_element(H, icr(I,C, R)),
			icr_set(T, icr(I1,C1, R1)),
			union(I,I1,I2),
			union(C,C1,C2),
			union(R,R1,R2).



%get the -ICR set for an +Element 
icr_set_element(E, ICR):-
	E= element(individual, [name=I],_),
	!,
	ICR=icr([I], [], []).

icr_set_element(E, ICR):-
	E= element(catom, [name=C],_),
      \+ stop_set(c, C),
	!,
	ICR=icr([], [C], []).

icr_set_element(E, ICR):-
	E= element(ratom, [name=R],_),
	!,
	ICR=icr([], [], [R]).

icr_set_element(E, ICR):-
	E= element(_, _X, L),
	!,
	icr_set(L, ICR).


icr_set_element(_,icr([],[],[])):-!.

% stop set defines a set of names which should be ignored on ICR sets.

stop_set(c, 'http://www.w3.org/2002/07/owl#Thing').
 
